import { deepCopy } from '../../utils/assist';

const convertColumnOrder = (columns, fixedType) => {
    let list = [];
    let other = [];
    columns.forEach((col) => {
        if (col.fixed && col.fixed === fixedType) {
            list.push(col);
        } else {
            other.push(col);
        }
    });
    return list.concat(other);
};

export { convertColumnOrder };

// set forTableHead to true when convertToRows, false in normal cases like table.vue
const getAllColumns = (cols, forTableHead = false) => {
    const columns = deepCopy(cols);
    const result = [];
    columns.forEach((column) => {
        if (column.children) {
            if (forTableHead) result.push(column);
            result.push.apply(result, getAllColumns(column.children, forTableHead));
        } else {
            result.push(column);
        }
    });
    return result;
};

export { getAllColumns };

const convertToRows = (columns, fixedType = false) => {
    const originColumns = fixedType ? fixedType === 'left' ? deepCopy(convertColumnOrder(columns, 'left')) : deepCopy(convertColumnOrder(columns, 'right')) : deepCopy(columns);
    let maxLevel = 1;
    const traverse = (column, parent) => {
        if (parent) {
            column.level = parent.level + 1;
            if (maxLevel < column.level) {
                maxLevel = column.level;
            }
        }
        if (column.children) {
            let colSpan = 0;
            column.children.forEach((subColumn) => {
                traverse(subColumn, column);
                colSpan += subColumn.colSpan;
            });
            column.colSpan = colSpan;
        } else {
            column.colSpan = 1;
        }
    };

    originColumns.forEach((column) => {
        column.level = 1;
        traverse(column);
    });

    const rows = [];
    for (let i = 0; i < maxLevel; i++) {
        rows.push([]);
    }

    const allColumns = getAllColumns(originColumns, true);

    allColumns.forEach((column) => {
        if (!column.children) {
            column.rowSpan = maxLevel - column.level + 1;
        } else {
            column.rowSpan = 1;
        }
        rows[column.level - 1].push(column);
    });

    return rows;
};

export { convertToRows };

const getRandomStr = function (len = 32) {
    const $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
    const maxPos = $chars.length;
    let str = '';
    for (let i = 0; i < len; i++) {
        str += $chars.charAt(Math.floor(Math.random() * maxPos));
    }
    return str;
};

export { getRandomStr };

const parseDate = (date) => {
    let reg
    if (date === null) return new Date(NaN) // Treat null as an invalid date
    if (date === undefined) return new Date()
    if (date instanceof Date) return date
    if ((typeof date === 'string' || typeof date === 'number') && (date + "").length === 10) {
        date = date * 1000;
        return new Date(date) // timestamp
    }
    // eslint-disable-next-line no-cond-assign
    if ((typeof date === 'string') &&
        (/.*[^Z]$/i.test(date)) // looking for a better way
        &&
        (reg = date.match(/^(\d{4})-?(\d{1,2})-?(\d{0,2})(.*?(\d{1,2}):(\d{1,2}):(\d{1,2}))?.?(\d{1,3})?$/))) {
        // 2018-08-08 or 20180808
        return new Date(
            reg[1], reg[2] - 1, reg[3] || 1,
            reg[5] || 0, reg[6] || 0, reg[7] || 0, reg[8] || 0
        )
    }
    return new Date(date) // timestamp
}

/**
 * 格式化时间
 * @param date
 * @param fmt
 * @returns {*}
 */
export const dateFormat = (fmt, d) => {
    const date = parseDate(d);
    if (!(date instanceof Date)) {
        return "";
    }
    var o = {
        'M+': date.getMonth() + 1, //月份
        'd+': date.getDate(), //日
        'h+': date.getHours(), //小时
        'H+': date.getHours(),
        'm+': date.getMinutes(), //分
        's+': date.getSeconds(), //秒
        'q+': Math.floor((date.getMonth() + 3) / 3), //季度
        'S': date.getMilliseconds() //毫秒
    };
    if (!isNotEmpty(fmt)) {
        fmt = 'yyyy-MM-dd hh:mm:ss';
    }
    if (/(y+)/.test(fmt)) {
        fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
    }
    for (var k in o) {
        if (new RegExp('(' + k + ')').test(fmt)) {
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
        }
    }
    return fmt;
}

/**
 * 是否为空
 * @param str
 * @returns {boolean}
 */
const isNotEmpty = (str) => {
    if (str !== '' && str !== null && typeof str !== 'undefined') {
        return true;
    }
    // console.warn('argument format is wrong');
    return false;
}

/*
* 判断obj是否为一个整数
*/
function isInteger(obj) {
    return Math.floor(obj) === obj;
}
/**
* 将一个浮点数转成整数，返回整数和倍数。如 3.14 >> 314，倍数是 100
* @param floatNum {number} 小数
* @return {object}
*   {times:100, num: 314}
*/
function toInteger(floatNum) {
    let ret = { times: 1, num: 0 }
    const isNegative = floatNum < 0;
    if (isInteger(floatNum)) {
        ret.num = floatNum;
        return ret;
    }
    const strfi = String(floatNum);
    const dotPos = strfi.indexOf('.');
    const len = strfi.substr(dotPos + 1).length;
    const times = Math.pow(10, len);
    let intNum = parseInt(Math.abs(floatNum) * times + 0.5, 10);
    ret.times = times;
    if (isNegative) {
        intNum = -intNum;
    }
    ret.num = intNum;
    return ret;
}

/**
* 核心方法，实现加减乘除运算，确保不丢失精度
* 思路：把小数放大为整数（乘），进行算术运算，再缩小为小数（除）
*
* @param {number} a 运算数1
* @param {number} b 运算数2
* @param {string} op 运算类型，有加减乘除（add/subtract/multiply/divide）
*
*/
function operation(a, b, op) {
    const o1 = toInteger(a);
    const o2 = toInteger(b);
    const n1 = o1.num;
    const n2 = o2.num;
    const t1 = o1.times;
    const t2 = o2.times;
    const max = t1 > t2 ? t1 : t2;
    let result = null;
    switch (op) {
        case 'add':
            if (t1 === t2) { // 两个小数位数相同
                result = n1 + n2
            } else if (t1 > t2) { // o1 小数位 大于 o2
                result = n1 + n2 * (t1 / t2)
            } else { // o1 小数位 小于 o2
                result = n1 * (t2 / t1) + n2
            }
            return result / max
        case 'subtract':
            if (t1 === t2) {
                result = n1 - n2
            } else if (t1 > t2) {
                result = n1 - n2 * (t1 / t2)
            } else {
                result = n1 * (t2 / t1) - n2
            }
            return result / max
        case 'multiply':
            result = (n1 * n2) / (t1 * t2)
            return result
        case 'divide':
            result = (n1 / n2) * (t2 / t1)
            return result
    }
}
// 加减乘除的四个接口
/**
 * 加法运算
 * @param {number} a
 * @param {number} b
 * @returns
 */
export const add = function (a, b) {
    return operation(Number(a), b, 'add')
}
/**
 * 减法运算
 * @param {number} a
 * @param {number} b
 * @returns
 */
export const subtract = function (a, b) {
    return operation(Number(a), b, 'subtract')
}
/**
 * 乘法运算
 * @param {number} a
 * @param {number} b
 * @returns
 */
export const multiply = function (a, b) {
    return operation(Number(a), b, 'multiply')
}
/**
 * 除法运算
 * @param {number} a
 * @param {number} b
 * @returns
 */
export const divide = function (a, b) {
    return operation(Number(a), b, 'divide')
}
